@@ -5,7 +5,7 @@ use Data::Printer::Object;
 use Data::Printer::Common;
 use Data::Printer::Config;
 
-our $VERSION = '0.99_020';
+our $VERSION = '0.99_021';
 
 my $rc_arguments;
 my %arguments_for;
@@ -28,24 +28,10 @@ sub import {
     }
     $arguments_for{$caller} = $args;
 
-    my $use_prototypes = exists $args->{use_prototypes}
-            ? $args->{use_prototypes}
-        : exists $rc_arguments->{$caller} && exists $rc_arguments->{$caller}{use_prototypes}
-            ? $rc_arguments->{$caller}{use_prototypes}
-        : exists $rc_arguments->{'_'}{use_prototypes}
-            ? $rc_arguments->{'_'}{use_prototypes}
-        : 1
-        ;
+    my $use_prototypes = _find_option('use_prototypes', $args, $caller, 1);
     my $exported = ($use_prototypes ? \&p : \&_p_without_prototypes);
 
-    my $imported = exists $args->{alias}
-            ? $args->{alias}
-        : exists $rc_arguments->{$caller} && exists $rc_arguments->{$caller}{alias}
-            ? $rc_arguments->{$caller}{alias}
-        : exists $rc_arguments->{'_'}{alias}
-            ? $rc_arguments->{'_'}{alias}
-        : 'p'
-        ;
+    my $imported = _find_option('alias', $args, $caller, 'p');
 
     { no strict 'refs';
         *{"$caller\::$imported"} = $exported;
@@ -67,11 +53,24 @@ sub np (\[@$%&];%) {
     my $caller = caller;
     my $args_to_use = _fetch_args_with($caller, \%properties);
     my $printer = Data::Printer::Object->new($args_to_use);
+
+    # force color level 0 on 'auto' colors:
+    if ($printer->colored eq 'auto') {
+        $printer->{_output_color_level} = 0;
+    }
+
     my $ref = ref $_[0];
     if ($ref eq 'ARRAY' || $ref eq 'HASH' || ($ref eq 'REF' && ref ${$_[0]} eq 'REF')) {
         $printer->{_refcount_base}++;
     }
-    return $printer->write_label . $printer->parse($_[0]);
+    my $output = $printer->parse($_[0]);
+    if ($printer->caller_message_position eq 'after') {
+        $output .= $printer->write_label;
+    }
+    else {
+        $output = $printer->write_label . $output;
+    }
+    return $output;
 }
 
 
@@ -83,13 +82,24 @@ sub p (\[@$%&];%) {
     my $caller = caller;
     my $args_to_use = _fetch_args_with($caller, \%properties);
     my $printer = Data::Printer::Object->new($args_to_use);
+    my $want_value = defined wantarray;
+    if ($printer->colored eq 'auto' && $printer->return_value eq 'dump' && $want_value) {
+        $printer->{_output_color_level} = 0;
+    }
+
     my $ref = ref $_[0];
     if ($ref eq 'ARRAY' || $ref eq 'HASH' || ($ref eq 'REF' && ref ${$_[0]} eq 'REF')) {
         $printer->{_refcount_base}++;
     }
-    my $output = $printer->write_label . $printer->parse($_[0]);
+    my $output = $printer->parse($_[0]);
+    if ($printer->caller_message_position eq 'after') {
+        $output .= $printer->write_label;
+    }
+    else {
+        $output = $printer->write_label . $output;
+    }
 
-    return _handle_output($printer, $output, !!defined wantarray, $_[0]);
+    return _handle_output($printer, $output, $want_value, $_[0]);
 }
 
 # This is a p() clone without prototypes. Just like regular Data::Dumper,
@@ -110,14 +120,26 @@ sub _p_without_prototypes  {
     my $caller = caller;
     my $args_to_use = _fetch_args_with($caller, \%properties);
     my $printer = Data::Printer::Object->new($args_to_use);
+
+    my $want_value = defined wantarray;
+    if ($printer->colored eq 'auto' && $printer->return_value eq 'dump' && $want_value) {
+        $printer->{_output_color_level} = 0;
+    }
+
     my $ref = ref( defined $item ? $item : $_[0] );
     if ($ref eq 'ARRAY' || $ref eq 'HASH' || ($ref eq 'REF'
         && ref(defined $item ? $item : ${$_[0]}) eq 'REF')) {
         $printer->{_refcount_base}++;
     }
-    my $output = $printer->write_label . $printer->parse((defined $item ? $item : $_[0]));
+    my $output = $printer->parse((defined $item ? $item : $_[0]));
+    if ($printer->caller_message_position eq 'after') {
+        $output .= $printer->write_label;
+    }
+    else {
+        $output = $printer->write_label . $output;
+    }
 
-    return _handle_output($printer, $output, !!defined wantarray, $_[0]);
+    return _handle_output($printer, $output, $want_value, $_[0]);
 }
 
 
@@ -146,14 +168,11 @@ sub _handle_output {
             return $data;
         }
     }
-    elsif ($printer->return_value eq 'void') {
+    elsif ($printer->return_value eq 'void' || !$wantarray) {
         print { $printer->output_handle } $output . "\n";
         return;
     }
     else {
-        if (!$wantarray) {
-            print { $printer->output_handle } $output . "\n";
-        }
         return $output;
     }
 }
@@ -185,6 +204,29 @@ sub _fetch_args_with {
     return $args_to_use;
 }
 
+sub _find_option {
+    my ($key, $args, $caller, $default) = @_;
+
+    my $value;
+    if (exists $args->{$key}) {
+        $value =  $args->{$key};
+    }
+    elsif (
+          exists $rc_arguments->{$caller}
+       && exists $rc_arguments->{$caller}{$key}
+    ) {
+        $value = $rc_arguments->{$caller}{$key};
+    }
+    elsif (exists $rc_arguments->{'_'}{$key}) {
+        $value = $rc_arguments->{'_'}{$key};
+    }
+    else {
+        $value = $default;
+    }
+    return $value;
+}
+
+
 'Marielle, presente.';
 __END__
 
@@ -429,6 +471,8 @@ active only on C<p()>/C<np()> calls made inside a given module:
     max_depth       = 1
     theme           = Monokai
     class.stringify = 0
+    # use quotes if you want spaces to be significant:
+    hash_separator  = " => "
 
     # You can set rules that apply only to a specific
     # caller module (in this case, MyApp::Some::Module):
@@ -481,7 +525,7 @@ information on each of them:
 
     # general options
     name           => 'var', # the name to use for circular references
-    return_value   => 'pass', # dump, void
+    return_value   => 'pass', # also 'dump' and 'void'
     output         => 'stderr', # stdout, \$string, $filehandle
     use_prototypes => 1,
     indent         => 4,
