@@ -50,23 +50,23 @@ sub _filter_category_for {
 # worry about, and every single piece of that could have its own color.
 # That, and hash keys and strings share this. So we put it all in one place.
 sub _process_string {
-    my ($ddp, $string, $src_name) = @_;
+    my ($ddp, $string, $src_color) = @_;
 
     # colorizing messes with reduce_string because we are effectively
     # adding new (invisible) characters to the string. So we need to
     # handle reduction first. But! Because we colorize string_max
     # *and* we should escape any colors already present, we need to
     # do both at the same time.
-    $string = _reduce_string($ddp, $string, $src_name);
+    $string = _reduce_string($ddp, $string, $src_color);
 
     # now we escape all other control characters except for "\e", which was
     # already escaped in _reduce_string(), and convert any chosen charset
     # to the \x{} format. These could go in any particular order:
-    $string = _escape_chars($ddp, $string, $src_name);
-    $string = _print_escapes($ddp, $string, $src_name);
+    $string = _escape_chars($ddp, $string, $src_color);
+    $string = _print_escapes($ddp, $string, $src_color);
 
     # finally, send our wrapped string:
-    return $ddp->maybe_colorize($string, $src_name);
+    return $ddp->maybe_colorize($string, $src_color);
 }
 
 sub _colorstrip {
@@ -328,7 +328,7 @@ sub _tryme {
     return ($error || '(unknown error)');
 }
 
-# adapted from File::HomeDir
+# adapted from File::HomeDir && File::HomeDir::Tiny
 sub _my_home {
     my ($testing) = @_;
     if ($testing) {
@@ -340,57 +340,31 @@ sub _my_home {
         mkdir($home, 0755) unless -d $home;
         return $home;
     }
-    elsif (exists $ENV{HOME} && defined $ENV{HOME}) {
-        return $ENV{HOME};
+    elsif ($^O eq 'MSWin32' and "$]" < 5.016) {
+        return $ENV{HOME} || $ENV{USERPROFILE};
     }
-    elsif ($^O eq 'MSWin32') {
-        return $ENV{USERPROFILE} if exists $ENV{USERPROFILE} and $ENV{USERPROFILE};
-        if (exists $ENV{HOMEDRIVE} and exists $ENV{HOMEPATH} and $ENV{HOMEDRIVE} and $ENV{HOMEPATH} ) {
-            require File::Spec;
-            return File::Spec->catpath($ENV{HOMEDRIVE}, $ENV{HOMEPATH}, '');
-        }
-        return undef;
-    }
-    elsif ($^O eq 'darwin' || $^O eq 'MacOS') {
-        my $error = _tryme(sub { require Mac::SystemDirectory; 1; });
+    elsif ($^O eq 'MacOS') {
+        my $error = _tryme(sub { require Mac::SystemDirectory; 1 });
         return Mac::SystemDirectory::HomeDirectory() unless $error;
-        $error = _tryme(sub { require Mac::Files; 1; });
-        if (!$error) {
-            my $folder = Mac::Files::FindFolder(
-                Mac::Files::kUserDomain(),
-                Mac::Files::kCurrentUserFolderType()
-            );
-            return undef unless defined $folder;
-            if (!-d $folder) {
-                # Make sure that symlinks resolve to directories.
-                return undef unless -l $folder;
-                my $dir = readlink $folder or return undef;
-                return undef unless -d $dir;
-            }
-            require Cwd;
-            return Cwd::abs_path($folder);
-        }
-        my $home;
-        SCOPE: { $home = (getpwuid($<))[7] }
-        if (defined $home && !-d $home) {
-            $home = undef;
-        }
-        return $home;
     }
-    else { # unix/linux/bsd/...
-        my $home;
-        if (exists $ENV{LOGDIR} and $ENV{LOGDIR}) {
-            $home = $ENV{LOGDIR};
-        }
-        else {
-            # Light desperation on any (Unixish) platform
-            SCOPE: { $home = (getpwuid($<))[7] }
-        }
-        if (defined $home and ! -d $home ) {
-            $home = undef;
-        }
-        return $home;
+    # this is the most common case, for most breeds of unix, as well as
+    # MSWin32 in more recent perls.
+    my $home = (<~>)[0];
+    return $home if $home;
+
+    # desperate measures that should never be needed.
+    if (exists $ENV{LOGDIR} and $ENV{LOGDIR}) {
+        $home = $ENV{LOGDIR};
     }
+    if (not $home and exists $ENV{HOME} and $ENV{HOME}) {
+        $home = $ENV{HOME};
+    }
+    # Light desperation on any (Unixish) platform
+    SCOPE: { $home = (getpwuid($<))[7] if not defined $home }
+    if (defined $home and ! -d $home ) {
+        $home = undef;
+    }
+    return $home;
 }
 
 # When printing array elements or hash keys, we may traverse all of it
@@ -467,7 +441,7 @@ sub _linear_ISA_for {
 }
 
 sub _initialize_mro {
-    my $error= _tryme(sub {
+    my $error = _tryme(sub {
         if ($] < 5.009_005) { require MRO::Compat }
         else { require mro }
         1;
