@@ -62,7 +62,8 @@ use Data::Printer::Filter::GenericClass;
 my @method_names =qw(
     name show_tainted show_unicode show_readonly show_lvalue show_refcount
     show_memsize memsize_unit print_escapes scalar_quotes escape_chars
-    caller_info caller_message string_max string_overflow string_preserve
+    caller_info caller_message caller_message_newline caller_message_position
+    string_max string_overflow string_preserve resolve_scalar_refs
     array_max array_overflow array_preserve hash_max hash_overflow
     hash_preserve ignore_keys unicode_charnames colored theme show_weak
     max_depth index separator end_separator class_method class hash_separator
@@ -141,6 +142,9 @@ sub _init {
                             'caller_message',
                             'Printing in line __LINE__ of __FILENAME__:'
                         );
+    $self->{'caller_message_newline'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'caller_message_newline', 1);
+    $self->{'caller_message_position'} = Data::Printer::Common::_fetch_anyof($props, 'caller_message_position', 'before', [qw(before after)]);
+    $self->{'resolve_scalar_refs'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'resolve_scalar_refs', 0);
     $self->{'string_max'} = Data::Printer::Common::_fetch_scalar_or_default($props, 'string_max', 1024);
     $self->{'string_preserve'} = Data::Printer::Common::_fetch_anyof(
                              $props,
@@ -221,9 +225,9 @@ sub _init {
         Data::Printer::Common::_fetch_scalar_or_default($props, 'fulldump', 0)
     );
 
+    $self->output(defined $props->{output} ? $props->{output} : 'stderr');
     $self->_load_colors($props);
     $self->_load_filters($props);
-    $self->output($props->{output} || 'stderr');
 
     my %extra_config;
     my %core_options = map { $_ => 1 }
@@ -298,12 +302,24 @@ sub multiline {
                 if exists $self->{_original_index};
             $self->hash_separator( $self->{_original_separator} )
                 if exists $self->{_original_separator};
+            $self->array_overflow( $self->{_original_array_overflow} )
+                if exists $self->{_original_array_overflow};
+            $self->hash_overflow( $self->{_original_hash_overflow} )
+                if exists $self->{_original_hash_overflow};
+            $self->string_overflow( $self->{_original_string_overflow} )
+                if exists $self->{_original_string_overflow};
         }
         else {
             $self->{_original_index} = $self->index;
             $self->index(0);
             $self->{_original_separator} = $self->hash_separator;
             $self->hash_separator(':');
+            $self->{_original_array_overflow} = $self->array_overflow;
+            $self->array_overflow('(...)');
+            $self->{_original_hash_overflow} = $self->hash_overflow;
+            $self->hash_overflow('(...)');
+            $self->{_original_string_overflow} = $self->string_overflow;
+            $self->string_overflow('(...)');
             $self->{_linebreak} = ' ';
             $self->{_current_indent} = 0;
         }
@@ -316,20 +332,20 @@ sub fulldump {
     if (defined $value) {
         $self->{fulldump} = !!$value;
         if ($value) {
-            $self->{_original_max_string} = $self->max_string;
-            $self->max_string(0);
-            $self->{_original_max_array} = $self->max_array;
-            $self->max_array(0);
-            $self->{_original_max_hash} = $self->max_hash;
-            $self->max_hash(0);
+            $self->{_original_string_max} = $self->string_max;
+            $self->string_max(0);
+            $self->{_original_array_max} = $self->array_max;
+            $self->array_max(0);
+            $self->{_original_hash_max} = $self->hash_max;
+            $self->hash_max(0);
         }
         else {
-            $self->max_string($self->{_original_max_string})
-                if exists $self->{_original_max_string};
-            $self->max_array($self->{_original_max_array})
-                if exists $self->{_original_max_array};
-            $self->max_hash($self->{_original_max_hash})
-                if exists $self->{_original_max_hash};
+            $self->string_max($self->{_original_string_max})
+                if exists $self->{_original_string_max};
+            $self->array_max($self->{_original_array_max})
+                if exists $self->{_original_array_max};
+            $self->hash_max($self->{_original_hash_max})
+                if exists $self->{_original_hash_max};
         }
     }
 }
@@ -415,7 +431,12 @@ sub _detect_color_level {
 
     # first we honour ANSI_COLORS_DISABLED, colored and tty piping
     if (   !$colored
-        || ($colored eq 'auto' && exists $ENV{ANSI_COLORS_DISABLED})
+        || ($colored eq 'auto'
+            && (exists $ENV{ANSI_COLORS_DISABLED}
+                || $self->output eq 'handle'
+                || $self->output eq 'file'
+            )
+        )
 #        || ! -t $self->{_output} # <-- FIXME TODO
     ) {
         $color_level = 0;
@@ -613,7 +634,13 @@ sub parse {
     # when parsing circular references:
     my $seen = $self->_see($data, %options);
     if (my $name = $seen->{name}) {
-        $parsed_string .= $self->maybe_colorize($name, 'repeated');
+        $parsed_string .= $self->maybe_colorize(
+            ((ref $data eq 'SCALAR' && $self->resolve_scalar_refs)
+                ? $$data
+                : $name
+            ),
+            'repeated'
+        );
         # on repeated references, the only extra data we put
         # is whether this reference is weak or not.
         $parsed_string .= $str_weak;
@@ -719,7 +746,7 @@ sub write_label {
     $message =~ s/\b__FILENAME__\b/$caller[1]/g;
     $message =~ s/\b__LINE__\b/$caller[2]/g;
 
-    return $message . "\n";
+    return $message . ($self->caller_message_newline ? "\n" : '');
 }
 
 sub maybe_colorize {
@@ -821,8 +848,8 @@ reference instead of entenring an infinite loop. However, there are cases
 when you wanto to print the same data structure twice, like when you're doing
 a second pass on a blessed object to print its internals, or if you're using
 the same object over and over again. This setting overrides the internal
-counter and prints the same data again. Check L<usee|/unsee> below for another
-way to achieve this.
+counter and prints the same data again. Check L<unsee|/unsee( $data )> below
+for another way to achieve this.
 
 =back
 
@@ -853,7 +880,7 @@ and position the "cursor" where you are expected to continue your dump string:
 Unless multiline was set to 0, the code above should print something like:
 
     this is a new line
-        this is indentend
+        this is indented
     back to our previous indentation
 
 =head2 maybe_colorize( $string, $label )
@@ -902,13 +929,15 @@ Most of them are described in L<Data::Printer>.
 
 =item * caller_message - what to print when caller_info is true.
 
+=item * caller_message_newline - skip line after printing caller_message (default: 1)
+
 =item * class - class properties to override.
 
 =item * class_method - function name to look for custom dump of external classes (default: '_dataprinter')
 
 =item * color_level - what the current color level is. Used by themes to approximate (or disable) colors.
 
-=item * colored - whether to colorize the output or not (default: 'auto')
+=item * colored - whether to colorize the output or not. Default is 'auto', meaning it will colorize only when printing to STDOUT or STDERR, never to a file or to a string (like when using np(). 'auto' also respects the ANSI_COLORS_DISABLED environment variable.
 
 =item * current_depth - shows the current depth level.
 
@@ -938,9 +967,9 @@ Most of them are described in L<Data::Printer>.
 
 =item * memsize_unit - show memory size as bytes (b), kbytes (k) or megabytes (m). Default is 'auto'
 
-=item * multiline - set to 0 to disable linebreaks, disable index and use ':' as hash separator.
+=item * multiline - defaults to 1. When set to 0, disables array index and linebreaks, uses ':' as hash separator and '(...)' as overflow for hashes, arrays and strings, and also disables 'caller_message_newline'.
 
-=item * fulldump - set to 1 to disable mas_string, max_array and max_hash at the same time.
+=item * fulldump - set to 1 to disable string_max, array_max and hash_max at the same time.
 
 =item * output - where the user wants the output to be printed. Defaults to 'stderr', could be 'stdout', \$string or $filehandle.
 
@@ -976,7 +1005,7 @@ Most of them are described in L<Data::Printer>.
 
 =item * sort_keys - whether to sort hash keys (default: 1)
 
-=item * string_max - maxumum number of characters in a string. Set to 0 to show all (default: 1024)
+=item * string_max - maximum number of characters in a string. Set to 0 to show all (default: 1024)
 
 =item * string_overflow - message to display once string_max is reached
 
